//
//              VPPDEMO.CPP
//              ===========
//
// Demonstration of VPE Professional Edition and above
//
// Copyright  1999 - 2008 IDEAL Software. All rights reserved.
//

#ifdef _WINDOWS
	#define STRICT
	#include <windows.h>
	#include <windowsx.h>
	#include <tchar.h>
	#include <io.h>
	#include <direct.h>
	#include "vppdemo.rh"
#else
	#include <unistd.h>
	#include <sys/stat.h>
	#include "VpeTypes.h"
	#define TCHAR		char
	#define _T(s)		(s)
	#define TRUE		1
	#define FALSE		0
	#define wsprintf	sprintf
#endif

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <time.h>
#include <locale.h>

#include "colors.h"
#include "vpecomon.h"
#include "vpiface.h"
#include "vppiface.h"


// ===============================================================
// Defines:
// ===============================================================
#define HELVETICA	"Helvetica"		// Windows: exchange with "Arial", if you want to use the True-Type font instead
#define TIMES		"Times"			// Windows: exchange with "Times New Roman", if you want to use the True-Type font instead

#if defined _MSC_VER && _MSC_VER >= 1400
	#pragma warning(disable:4996)	// fopen_s, etc. secure functions
#endif

// #define CREATE_FILES    // if defined, all Docs are written to file


#define PRODUCT "Virtual Print Engine v%d.%d Demo"


// ===============================================================
// Globals:
// ===============================================================
#ifdef _WINDOWS
	HWND  hMainWindow;
	HMENU hMenu;
#else
	void *hMainWindow = NULL;
#endif

char  szAppName[] = "VPE Professional Demo 3";
bool  AddedCharts;
VpeCoord LineHeight;         // used for Rendering

VpeHandle RTF = 0;
VpeHandle Clickable = 0;
VpeHandle Bar2D = 0;
VpeHandle UDO = 0;
VpeHandle Scale2Gray = 0;
VpeHandle PicExport = 0;
VpeHandle hDocImport = 0;

struct lconv Locale;


// ===============================================================
//                               Msg
// ===============================================================
void Msg(char *s)
{
#ifdef _WINDOWS
	MessageBox(hMainWindow, s, szAppName, MB_OK);
#else
	printf("%s\n", s);
#endif
}


// ========================================================================
//                               WinMsg
// For Windows only
// ========================================================================
void WinMsg(char *s)
{
#ifdef _WINDOWS
	MessageBox(hMainWindow, s, szAppName, MB_OK);
#endif
}


// ========================================================================
//                              MakeFileName
//
// On Non-Windows platforms, convert the supplied file name from Windows
// format to Non-Windows format.
// ========================================================================
const char *MakeFileName(const char *fname)
{
#ifdef _WINDOWS
	return fname;
#else
	if (!fname)
		return NULL;

	// Changes all "\\" into "/" of the string in place.
	static char new_name[4096];
	strcpy(new_name, fname);
	size_t len = strlen(new_name);

	for (size_t i = 0; i < len; i++)
	{
		if (new_name[i] == _T('\\'))
			new_name[i] = _T('/');
	}

	return new_name;
#endif
}


// ************************************************************************
//							MEMORY STREAMS
//
// The following functions deal with memory streams.
// This code is not used in the demo, but you can easily activate it by
// calling the function "StreamDemo()" from any of the functions within this
// source code, which creates a document (for expample at the end of the
// function "RTFDemo()".
//
// All the memory stream demos work with disk based files, which doesn't
// really make sense. Normally, you would read / write the memory streams
// as BLOBs from/to a database or transmit them via network (e.g. send a
// PDF file via HTTP to a client browser) or something like that.
// ************************************************************************

// ========================================================================
//                              ReadStreamFromFile
//
// Reads an external file from disk into a memory stream and returns the
// handle to the memory stream.
// ========================================================================
VpeHandle ReadStreamFromFile(VpeHandle hDoc, LPCTSTR import_file_name)
{
	// Create a memory stream
	VpeHandle hStream = VpeCreateMemoryStream(hDoc, 0);
	if (!hStream)
	{
		Msg("Error creating memory stream, out of memory!");
		return NULL;
	}

	// Import an external file to the memory stream.
	// This is only for demonstration purposes, normally you would read the memory stream now
	// as BLOB from a database or do something like that.
	FILE *fh = fopen(import_file_name, "rb");
	if (!fh)
	{
		Msg("Error importing the external file, can not open file!");
		VpeCloseStream(hStream);
		return NULL;
	}

	// As long as there is data, copy it
	while (!feof(fh))
	{
		const long BUFSIZE = 4096;
		BYTE buffer[BUFSIZE];

		// Read data from the file into a memory buffer
		long bytes_read = fread(buffer, 1, BUFSIZE, fh);

		// Write the memory buffer to the VPE stream
		VpeStreamWrite(hStream, buffer, bytes_read);
		if (!VpeGetStreamState(hStream))
		{
			Msg("Error importing the external file, out of memory!");
			VpeCloseStream(hStream);
			fclose(fh);
			return NULL;
		}
	}

	fclose(fh);

	// seek to the beginning of the memory file, so any successive read-operation starts at the beginning
	VpeStreamSeek(hStream, 0);

	return hStream;
}


// ========================================================================
//                              WriteMemoryStream
//
// Write a given stream to an external file on disk with the  given name.
// ========================================================================
void WriteMemoryStream(VpeHandle hDoc, VpeHandle hStream, LPCTSTR export_file_name)
{
	// Export the memory stream to an external file.
	// This is only for demonstration purposes, normally you would write the memory stream now
	// as BLOB to a database or do something like that.
	FILE *fh = fopen(export_file_name, "wb");
	if (!fh)
	{
		Msg("Error exporting the memory stream to external file, can not create file!");
		VpeCloseStream(hStream);
		return;
	}

	// Seek to the beginning of the memory file
	VpeStreamSeek(hStream, 0);

	// As long as there is data, copy it
	while (!VpeStreamIsEof(hStream))
	{
		const long BUFSIZE = 4096;
		BYTE buffer[BUFSIZE];

		// Read data from the VPE stream into a memory buffer
		long bytes_read = VpeStreamRead(hStream, buffer, BUFSIZE);

		// Write the memory buffer to the external file
		if (fwrite(buffer, 1, bytes_read, fh) != (size_t)bytes_read)
		{
			Msg("Error exporting the memory stream to external file, can not write to file!");
			VpeCloseStream(hStream);
			fclose(fh);
			return;
		}
	}

	fclose(fh);

	// Finally, close the memory stream. This is IMPORTANT, otherwise you will have memory-leaks!
	VpeCloseStream(hStream);
}


// ========================================================================
//                              WriteDocToMemoryStream
//
// Write the given document to a memory stream and from there to an
// external file on disk with the given name.
// ========================================================================
void WriteDocToMemoryStream(VpeHandle hDoc, int doc_type, int from_page, int to_page, LPCTSTR export_file_name)
{
	// Create a memory stream
	VpeHandle hStream = VpeCreateMemoryStream(hDoc, 0);
	if (!hStream)
	{
		Msg("Error creating memory stream, out of memory!");
		return;
	}

	// Select the desired export document type
	VpeSetDocExportType(hDoc, doc_type);

	VpeSetSubject(hDoc, "Tutorial for VPE");
	VpeSetAuthor(hDoc, "IDEAL Software GmbH");
	VpeSetKeywords(hDoc, "Tutorial VPE");
	VpeSetCreator(hDoc, "VPE Demo");

	VpeSetEncryption(hDoc, 1);
	VpeSetProtection(hDoc, PDF_ALLOW_PRINT + PDF_ALLOW_HIQ_PRINT);
	VpeSetEmbedAllFonts(hDoc, 1);
	VpeSetSubsetAllFonts(hDoc, 1);
	VpeSetFastWebView(hDoc, 1);

	// Write the document to the memory stream
	if (!VpeWriteDocStreamPageRange(hDoc, hStream, from_page, to_page))
	{
		Msg("Error writing to memory stream, out of memory!");
		VpeCloseStream(hStream);
		return;
	}

	WriteMemoryStream(hDoc, hStream, export_file_name);
}


// ========================================================================
//                              StreamDemo
// ========================================================================
void StreamDemo(VpeHandle hDoc)
{
	// Read the file "cap.vpe" into a memory stream and append it to the given document
	VpeHandle hStream = ReadStreamFromFile(hDoc, MakeFileName("..\\images\\cap.vpe"));

	// Read the VPE document from the memory stream
	if (hStream)
	{
		if (!VpeReadDocStreamPageRange(hDoc, hStream, 1, 14))
			Msg("Error reading from memory stream!\n\n"
				"Possible reasons:\n"
				"- Incompatible document version\n"
				"- File is not a VPE document file\n"
				"- File damaged");

		// Finally, close the memory stream. This is IMPORTANT, otherwise you will have memory-leaks!
		VpeCloseStream(hStream);
	}

	// Add a new page and import an image from a stream
	VpePageBreak(hDoc);
	hStream = ReadStreamFromFile(hDoc, MakeFileName("..\\images\\LOGO.BMP"));
	VpePictureStream(hDoc, hStream, 1.00, 1.00, VFREE, VFREE, MakeFileName("..\\images\\LOGO.BMP"));
	VpeCloseStream(hStream);

	// Export the documents to streams (and then the streams to disk files)
	WriteDocToMemoryStream(hDoc, VPE_DOC_TYPE_PDF, 1, 1, "mem.pdf");
	WriteDocToMemoryStream(hDoc, VPE_DOC_TYPE_VPE, 1, 1, "vpe_range.vpe");

	// Picture Export
#ifdef _WINDOWS
	hStream = VpeCreateMemoryStream(hDoc, 0);
	if (!hStream)
	{
		Msg("Error creating memory stream, out of memory!");
		return;
	}
	VpeSetPictureExportColorDepth(hDoc, PICEXP_COLOR_TRUE);
	VpeSetPictureDefaultDPI(hDoc, 300, 300);
	VpeSetPictureType(hDoc, PIC_TYPE_JPEG);
	VpePictureExportPageStream(hDoc, hStream, 1);
	WriteMemoryStream(hDoc, hStream, "mem_picexp.jpg");	// calls VpeCloseStream()!!!

	hStream = VpeCreateMemoryStream(hDoc, 0);
	if (!hStream)
	{
		Msg("Error creating memory stream, out of memory!");
		return;
	}
	VpeSetPictureType(hDoc, PIC_TYPE_WMF);
	VpePictureExportPageStream(hDoc, hStream, 1);
	WriteMemoryStream(hDoc, hStream, "mem_picexp.wmf");

	hStream = VpeCreateMemoryStream(hDoc, 0);
	if (!hStream)
	{
		MessageBox(NULL, "Error creating memory stream, out of memory!", "", MB_OK);
		return;
	}
	VpeSetPictureType(hDoc, PIC_TYPE_EMF);
	VpePictureExportPageStream(hDoc, hStream, 1);
	WriteMemoryStream(hDoc, hStream, "mem_picexp.emf");
#endif
}


// ========================================================================
//							  GetTmpDir
// ========================================================================
TCHAR *GetTmpDir()
{
#ifdef _WINDOWS
	TCHAR *s;

	s = _tgetenv(_T("TMP"));
	if (!s)
		s = _T("c:\\tmp");

	if (_taccess(s, 2))
	{
		s = _tgetenv(_T("TEMP"));
		if (!s)
			s = _T("c:\\temp");

		if (_taccess(s, 2))
		{
			s = _tgetenv(_T("ALLUSERSPROFILE"));
			if (!s)
				s = _T("c:");
		}
	}

	return s;

#else
	char *s;
	s = getenv("HOME");
	if (!s)
		s = "/tmp";
	return s;
#endif
}


// ========================================================================
//                         ExportDocument
// ========================================================================
void ExportDocument(VpeHandle hDoc, TCHAR* FName)
{
	VpeSetSubject(hDoc, "Tutorial for VPE");
	VpeSetAuthor(hDoc, "IDEAL Software GmbH");
	VpeSetKeywords(hDoc, "Tutorial VPE");
	VpeSetCreator(hDoc, "VPE Demo");

	VpeSetEncryption(hDoc, 1);
	VpeSetProtection(hDoc, PDF_ALLOW_PRINT + PDF_ALLOW_HIQ_PRINT);
	VpeSetEmbedAllFonts(hDoc, 1);
	VpeSetSubsetAllFonts(hDoc, 1);
	VpeSetFastWebView(hDoc, 1);

#ifdef _WINDOWS
	HWND hWnd = VpeGetWindowHandle(hDoc);
	TCHAR dir[MAX_PATH];
	wsprintf(dir, _T("%s\\%s"), GetTmpDir(), FName);

  #ifndef PROFILE
	if (MessageBox(hWnd, "Create and show PDF?", szAppName, MB_YESNO | MB_ICONQUESTION) == IDYES)
  #endif
	{
		_int64 start;
		QueryPerformanceCounter((LARGE_INTEGER *)&start);

		if (VpeWriteDoc(hDoc, dir) == TRUE)
		{
  #ifndef PROFILE
			_int64 now, freq;
			QueryPerformanceCounter((LARGE_INTEGER *)&now);
			QueryPerformanceFrequency((LARGE_INTEGER *)&freq);

			char Msg[255];
			sprintf(Msg, "Elapsed Time: %f seconds", ((double)(now - start) / (double)freq));
			MessageBox(hWnd, Msg, "Message", MB_OK);

			MessageBox(hWnd, "PDF Document successfully created!", szAppName, MB_OK | MB_ICONINFORMATION);
			ShellExecute(0, NULL, dir, NULL, NULL, SW_SHOWMAXIMIZED);
  #else
			PostMessage(hMainWindow, WM_CLOSE, 0, 0);
  #endif
		}
		else
		{
              MessageBox(hWnd, "Error creating PDF file!\n"
              "Possible reasons:\n"
              " - the file is open in Acrobat Reader\n"
              " - hard disk full\n"
              " - no access rights to the folder\n"
              " - out of memory\n"
              " - export module missing", szAppName, MB_OK | MB_ICONERROR);
		}
	}
#else
	// Non-Windows
	char dir[4096];
	sprintf(dir, _T("%s/%s"), GetTmpDir(), FName);

	printf("creating file %s\n", dir);
	if (!VpeWriteDoc(hDoc, dir))
	{
		printf("Error creating PDF file!\n"
			"Possible reasons:\n"
			" - the file is open in a PDF Reader\n"
			" - hard disk full\n"
			" - no access rights to the folder\n"
			" - out of memory\n");
	}
#endif
}


// ========================================================================
//                            NumberString
//
// Create a number string depending on the actual country settings in the
// control panel.
//
// Expects the decimal point in the parameter string as "." !
//
// RETURNS:   The string passed as parameter is directly modified.
//
// ATTENTION: Therefore do not pass a string from the Constant Data Segment
//            to this function directly,
//            e.g. char *s = "1234";
//                 NumberString(s);    is forbidden!
// ========================================================================
void NumberString(char *s)
{
   static char cDecimal = 0;
   static char sThousand[2];

   size_t len, begin, tocopy;
   char *p, tmp[128];

   if (!s)
      return;
      
   strcpy(tmp, s);

   if (!cDecimal)
   {
      cDecimal = *Locale.mon_decimal_point;
	  sThousand[0] = *Locale.mon_thousands_sep;
	  sThousand[1] = '\0';
   }
   
#ifndef _WINDOWS
   // On some systems the setlocale() call does not make (s)printf() format the double value as desired,
   // so replace decimal point:
   p = tmp;
   while (*p)
   {
      if (*p == '.')
      {
         *p = cDecimal;
         break;         // there is only 1 decimal point max. --> exit loop
      }
      p++;
   }
#endif

   // split fractional part off:
   // ==========================
   p = strchr(tmp, cDecimal);
   len = strlen(tmp);
   if (p)
      len -= len - (p - tmp);
   
   p = tmp;
   begin = TRUE;
   *s = '\0';

   // insert thousand-points:
   // ======================= 
   while (len > 0)
   {
      tocopy = len % 3;
      if (tocopy == 0)
      {
         if (!begin)
            strcat(s, sThousand);
         tocopy = 3;
      }
      strncat(s, p, tocopy);
      p += tocopy;
      len -= tocopy;
      begin = FALSE;
   }
 
   strcat(s, p);
}



void double2NumberString(double d, char *s)
{
   sprintf(s, "%.2f", d);
   NumberString(s);
}


// ===============================================================
//                           RTFDemo
// ===============================================================

#ifdef PROFILE
// This function is only for internal profiling issues
void RTFMeasure()
{
   VpeHandle hDoc = VpeOpenDoc(hMainWindow, "RTF Demo", VPE_GRIDBUTTON | VPE_NO_HELPBUTTON | VPE_NO_INFOBUTTON | VPE_EMBEDDED);

   VpeReadDoc(hDoc, "rtf.vpe");
   ExportDocument(hDoc, "Rtf.pdf");
}
#endif


void RTFDemo()
{
   char s[80];
   
   WinMsg("This demo shows the RTF support of VPE Professional Edition.\n\n"
          "It will read now the file '..\\images\\DMODOC32.RTF' (16-bit: 'DMODOC16.RTF') into memory and display it.\n"
          "Of course you can also create RTF texts on-the-fly directly by code without reading files.\n\n"
          "Please note how fast and accurate VPE works!");

   VpeHandle hDoc = VpeOpenDoc(hMainWindow, "RTF Demo", VPE_GRIDBUTTON | VPE_NO_HELPBUTTON | VPE_NO_INFOBUTTON | VPE_EMBEDDED);
	if (!hDoc)
	{
		Msg("The Document could not be created!");
		return;
	}
		
   RTF = hDoc;

#ifdef _WINDOWS
   VpePreviewDoc(hDoc, NULL, VPE_SHOW_NORMAL);

   // this is an embedded window, forward the focus to the preview, so
   // it receives keyboard and mouse-wheel messages
   SetFocus(VpeGetWindowHandle(hDoc));
#else
   // The Rich Text file is using True-Type fonts. On non-Windows and non-Mac platformswe don't know, if
   // the required True-Type fonts are available. It is your resonsibility - as the application developer - to
   // install (or make sure) that required fonts are available on the target machine.
   // We are using here Font Substitution to overcome this problem. The Rich Text file is using the True-Type
   // fonts: Arial, Times New Roman, Symbol and Wingdings.
   // We will substitute:
   // - Arial with the PostScript font Helvetica
   // - Times New Roman with the PostScript font Times
   // - Wingdings with Symbol (note that these fonts are NOT identical!)
   VpeSetFontSubstitution(hDoc, "Arial", "Helvetica");
   VpeSetFontSubstitution(hDoc, "Times New Roman", "Times");
   VpeSetFontSubstitution(hDoc, "Wingdings", "Symbol");
#endif

   // Use the original Document Settings
   VpeSet(hDoc, VLEFTMARGIN, 2.54);
   VpeSet(hDoc, VTOPMARGIN, 2.50);
   VpeSet(hDoc, VRIGHTMARGIN, 19.10);
   VpeSet(hDoc, VBOTTOMMARGIN, 27.20);
   VpeSetDefOutRect(hDoc, 2.54, 2.50, 19.10, 27.20);

   VpeWriteRTFFile(hDoc, VLEFTMARGIN, VTOPMARGIN, VRIGHTMARGIN, VFREE, MakeFileName("..\\images\\DMODOC32.RTF"));

   // Insert page numbers in the style "Page n of <total>":
   // =====================================================
   VpeSetTextAlignment(hDoc, ALIGN_RIGHT);
   VpeSetItalic(hDoc, TRUE);
   VpeSetPenSize(hDoc, 0.06);
   for (int page = 2; page <= VpeGetPageCount(hDoc); page++)
   {
      VpeGotoPage(hDoc, page);
      sprintf(s, "Page %d of %d", page, VpeGetPageCount(hDoc));
      VpeWrite(hDoc, VpeGet(hDoc, VRIGHTMARGIN) - 5.00, 1.25, VpeGet(hDoc, VRIGHTMARGIN), VFREE, s);
      VpeLine(hDoc, 2.54, VpeGet(hDoc, VBOTTOM) + 0.08, 19.10, VpeGet(hDoc, VBOTTOM) + 0.08);
   }

  #ifdef CREATE_FILES
   #pragma message("*** CREATE FILES ***")
   unlink("rtf.vpe");
   VpeWriteDoc(hDoc, "rtf.vpe");
  #endif

   ExportDocument(hDoc, "Rtf.pdf");
}





// ===============================================================
//                            Clickable
// ===============================================================
#define DATA_COUNT      4
#define FACTOR          49

struct _tagTable
{
   char     *product;
   double   germany;
   double   france;
   double   usa;
   double   australia;
   COLORREF chart_color;  // for Overview Chart
   VpeHandle hDoc;        // associated document handle, for Chart-Preview
} table[DATA_COUNT] =
{
   "Apples",  2450, 1700, 3200, 1080, COLOR_GREEN,      0,
   "Oranges", 3650, 1470, 2830, 2240, COLOR_ORANGE,     0,
   "Bananas",  900, 2130,  520, 1560, COLOR_LTYELLOW,   0,
   "Overview",   0,    0,    0,    0, COLOR_LTYELLOW,   0,  // for "Overview Chart"
};




void make_chart(VpeHandle hDoc, VpeCoord x, VpeCoord y, int index)
{
   VpeHandle hData;
   
   if (index < DATA_COUNT - 1)
   {
      // Create one of the three Pie Charts
      // ==================================
      hData = VpeChartDataCreate(hDoc, 4, 1);
      VpeSetChartTitle(hDoc, table[index].product);

      VpeChartDataAddValue(hDoc, hData, 0, table[index].germany);
      VpeChartDataAddValue(hDoc, hData, 1, table[index].france);
      VpeChartDataAddValue(hDoc, hData, 2, table[index].usa);
      VpeChartDataAddValue(hDoc, hData, 3, table[index].australia);
      VpeChartDataAddLegend(hDoc, hData, "Germany");
      VpeChartDataAddLegend(hDoc, hData, "France");
      VpeChartDataAddLegend(hDoc, hData, "USA");
      VpeChartDataAddLegend(hDoc, hData, "Australia");

    #if defined(__WIN32__) || defined(WIN32) || defined(__NT__)
    #else
      if (hDoc == PicExport)
      {
         // Use darker colors in Win16 for better dithering:
         VpeChartDataSetColor(hDoc, hData, 0, COLOR_RED);
         VpeChartDataSetColor(hDoc, hData, 1, COLOR_DKBLUE);
         VpeChartDataSetColor(hDoc, hData, 2, COLOR_DKYELLOW);
         VpeChartDataSetColor(hDoc, hData, 3, COLOR_DKGREEN);
      }
   #endif


      VpeSetBkgMode(hDoc, VBKG_SOLID);
      VpeSetBkgColor(hDoc, COLOR_LTGRAY);
      VpeChart(hDoc, x, y, -12.00, -12.00, hData, VCHART_3D_PIE);
   }
   else
   {
      // Create the Total Chart
      // ======================
      hData = VpeChartDataCreate(hDoc, DATA_COUNT - 1, 4);
      VpeSetChartTitle(hDoc, "Overview");
      VpeSetTextColor(hDoc, COLOR_MAGENTA);
      VpeSetChartXLabelState(hDoc, VCHART_LABEL_USER);
      VpeChartDataAddXLabel(hDoc, hData, "Germany");
      VpeChartDataAddXLabel(hDoc, hData, "France");
      VpeChartDataAddXLabel(hDoc, hData, "USA");
      VpeChartDataAddXLabel(hDoc, hData, "Australia");
      for (int n = 0; n < DATA_COUNT - 1; n++)
      {
         VpeChartDataAddLegend(hDoc, hData, table[n].product);
         VpeChartDataSetColor(hDoc, hData, n, table[n].chart_color);
         VpeChartDataAddValue(hDoc, hData, n, table[n].germany);
         VpeChartDataAddValue(hDoc, hData, n, table[n].france);
         VpeChartDataAddValue(hDoc, hData, n, table[n].usa);
         VpeChartDataAddValue(hDoc, hData, n, table[n].australia);
      }

      VpeSetBkgMode(hDoc, VBKG_GRD_LINE);
      VpeSetBkgGradientEndColor(hDoc, COLOR_BLUE);
      VpeChart(hDoc, x, y, -12.00, -12.00, hData, VCHART_3D_STACKED_BAR_ABSOLUTE);
   }
}




int AddCharts(VpeHandle hDoc)
{            
   // Add Charts to document
   for (int n = 0; n < DATA_COUNT; n++)
   {
      if ((n + 1) % 2)
         VpePageBreak(hDoc);
      make_chart(hDoc, 2.00, 1.00 + 14.00 * (n % 2), n);

	  // Write annotation
	  VpeStoreSet(hDoc, 1);
      VpeSetPenSize(hDoc, 0.03);
      VpeSetTextColor(hDoc, COLOR_BLACK);
      VpeSetCornerRadius(hDoc, 0.50);
      VpeSetBkgMode(hDoc, VBKG_GRD_LINE);
      VpeSetBkgGradientTriColor(hDoc, TRUE);
      VpeSetBkgGradientMiddleColorPosition(hDoc, 40);
      VpeSetBkgGradientMiddleColorPosition(hDoc, 50);
      VpeSetBkgGradientStartColor(hDoc, COLOR_ORANGE);
      VpeSetBkgGradientEndColor(hDoc, COLOR_ORANGE);
      VpeSetShadowed(hDoc, TRUE);
      VpeWriteBox(hDoc, 15.00, 1.00 + 14.00 * (n % 2), VFREE, VFREE, "   Annotation   ");
  	  VpeUseSet(hDoc, 1);
  	  VpeRemoveSet(hDoc, 1);
   }

   return TRUE;
}


#ifdef _WINDOWS
void show_chart(int index)
{
   VpeHandle hDoc;
   
   index--;
   if (index < DATA_COUNT)
   {
      if (table[index].hDoc)
      {
         // Chart already previewed, simply set the focus to the document and exit:
         SetFocus(VpeGetWindowHandle(table[index].hDoc));
         return;
      }
      
      // Create Chart Document
      // =====================
      hDoc = VpeOpenDoc(hMainWindow, table[index].product, VPE_NO_RULERS +
                                                           VPE_NO_STATBAR +
                                                           VPE_NO_MOVEBTNS +
                                                           VPE_NO_SCALEBTNS);
      table[index].hDoc = hDoc;
      VpeNoPen(hDoc);
      make_chart(hDoc, 0, 0, index);
      MoveWindow(VpeGetWindowHandle(table[index].hDoc), index * 30, index * 30, 461, 505, FALSE);
      VpeSetPreviewWithScrollers(hDoc, FALSE);
      VpeSetPaperView(hDoc, FALSE);
      VpeSetPrintOffset(hDoc, 2.00, 2.00);
	  VpeSetScalePercent(hDoc, 100);
      VpePreviewDoc(hDoc, NULL, VPE_SHOW_NORMAL);
   }
}



int CloseAllCharts()
{
	int retval = TRUE;
	
	// Close all Chart Previews:
   // =========================
   for (int i = 0; i < DATA_COUNT; i++)
   {
      if (!VpeCloseDoc(table[i].hDoc))
         retval = FALSE;
   }

	return retval;
}
#endif


void GroupHeader(VpeHandle hDoc, char *Product)
{
   VpeCoord y;
   char s[80];

   VpeSetTextAlignment(hDoc, ALIGN_LEFT);
   
   VpeSetTextColor(hDoc, COLOR_BLUE);
   wsprintf(s, "[N S 18]Product: %s", Product);
   
   y = VpePrint(hDoc, 1.50, VpeGet(hDoc, VBOTTOM), s);
   y = y + 0.10;

   // Avoid 16-bit picture export color problem:
   VpeSetTextColor(hDoc, COLOR_PURPLE);
   VpePrint(hDoc, 1.50, y, "[S 16]Country");
   VpeWrite(hDoc, 6.50, y, 9.00, VFREE, "[R]Quantity");
   VpeWrite(hDoc, 11.50, y, 14.50, VFREE, "Value (in $)");
   VpeSetTextColor(hDoc, COLOR_BLACK);
}



void DetailLine(VpeHandle hDoc, COLORREF Color, char *Country, double quantity)
{
   VpeCoord y;
   char s[80];

   y = VpeGet(hDoc, VBOTTOM);
   VpeSetBkgMode(hDoc, FALSE);
   VpeSetBkgColor(hDoc, Color);
   VpeSetPenColor(hDoc, Color);
   VpeBox(hDoc, 1.50, y, 15.50, LineHeight);
   VpeSetBkgMode(hDoc, TRUE);

   wsprintf(s, "[S 14 C Black L]%s", Country);
   VpeWriteBox(hDoc, 1.50, y, -4.00, LineHeight, s);
   VpeSetTextAlignment(hDoc, ALIGN_RIGHT);
   double2NumberString(quantity, s);
   VpeWriteBox(hDoc, 6.50, y, 9.00, LineHeight, s);
   double2NumberString(quantity * FACTOR, s);
   VpeWriteBox(hDoc, 11.50, y, 14.50, LineHeight, s);
}



void GroupFooter(VpeHandle hDoc, double quantity)
{
   VpeCoord y;
   char s[80];

   // PrintBox and WriteBox is used here, because in the Detail-Section they are used, too.
   // If we wouldn't use them, the Footer-Section would be misaligned, because of
   // the invisible surrounding rectangle of the Detail-Section.
   y = VpeGet(hDoc, VBOTTOM) + VpeGetPenSize(hDoc) + 0.02;
                // + Pensize, because the white frame would otherwise overlap the Detail-Section's frame
                // +2, to avoid rounding problems we use a bit more
   VpeSetPenColor(hDoc, COLOR_WHITE);
   VpePrintBox(hDoc, 1.50, y, "Total");
   double2NumberString(quantity, s);
   VpeWriteBox(hDoc, 6.50, y, 9.00, LineHeight, s);
   double2NumberString(quantity * FACTOR, s);
   VpeWriteBox(hDoc, 11.50, y, 14.50, LineHeight, s);
   VpeSet(hDoc, VBOTTOM, y + 1.80);
}

#ifdef _WINDOWS
void clickable(VpeHandle hDoc = 0, bool pic_export = false)
{
   VpeCoord y;

   if (!pic_export)
   {
      AddedCharts = false;
      Msg("This demo shows, how Clickable Objects and Charts work with VPE Professional Edition.\n"
          "Click onto the tables and onto the total sum to see different charts.");
      hDoc = VpeOpenDoc(hMainWindow, "Clickable Report", VPE_GRIDBUTTON | VPE_NO_HELPBUTTON | VPE_NO_INFOBUTTON);
      Clickable = hDoc;
   }

   VpeSetFontName(hDoc, HELVETICA);
   VpeRenderPrintBox(hDoc, 0, 0, "[S 14 C Black]x");
   LineHeight = -VpeGet(hDoc, VRENDERHEIGHT);
   
   VpeSetAutoBreak(hDoc, AUTO_BREAK_NO_LIMITS);
   VpeSetPenSize(hDoc, 0.05);
   if (!pic_export)
      VpeSetObjectID(hDoc, 100);
   VpePicture(hDoc, 16.50, 1.50, VFREE, VFREE, MakeFileName("..\\images\\FRUITS.BMP"));
   VpeWriteBox(hDoc, 16.50, VpeGet(hDoc, VBOTTOM), VpeGet(hDoc, VRIGHT), VFREE, "[N S 9 CE I C Blue BC Gray TO]Better Fruits Software");
   VpeSetObjectID(hDoc, 0);
   y = VpeGet(hDoc, VBOTTOM) + 0.50;
   VpePrint(hDoc, 1.50, 2.00, "[N S 26 U]Year End Results");

   VpePrint(hDoc, 1.50, 4.00, "[N S 32]Better Fruits Software");

   VpeLine(hDoc, 1.50, y, 20.00, y);
   VpeSet(hDoc, VBOTTOM, y + 0.50);
   VpeSetPenSize(hDoc, 0.03);

   double group_total, total = 0;
   char s[80], s2[80];
   int n;

   for (n = 0; n < DATA_COUNT - 1; n++)
   {
      if (!pic_export)
         VpeSetObjectID(hDoc, n+1);
      GroupHeader(hDoc, table[n].product);
      DetailLine(hDoc, COLOR_GRAY, "Germany", table[n].germany);
      DetailLine(hDoc, COLOR_HIBLUE, "France", table[n].france);
      DetailLine(hDoc, COLOR_GRAY, "USA", table[n].usa);
      DetailLine(hDoc, COLOR_HIBLUE, "Australia", table[n].australia);
      group_total = table[n].germany +
                    table[n].france +
                    table[n].usa +
                    table[n].australia;
      GroupFooter(hDoc, group_total);
      total += group_total;
   }

   if (!pic_export)
      VpeSetObjectID(hDoc, n + 1);
   double2NumberString(total * FACTOR, s);
   sprintf(s2, "[N S 20 U]Yearly Country Sales Total: $%s", s);
   y = VpePrint(hDoc, 1.50, VpeGet(hDoc, VBOTTOM), s2);

   VpeSetPrintable(hDoc, FALSE);
   VpeSetStreamable(hDoc, FALSE);
   VpeSetShadowed(hDoc, TRUE);

   if (!pic_export)
   {
      VpeSetObjectID(hDoc, 101);
      VpePrintBox(hDoc, 1.50, y + 0.40, "[N S 12 I B PC LTYELLOW TO BC LTYELLOW]Click here to add the charts to the document!");
   }
   else
   {
      VpeSetObjectID(hDoc, 200);
      VpePrintBox(hDoc, 1.50, y + 0.40, "[N S 12 I B PC LTYELLOW TO BC LTYELLOW]Click here to export the document to image files!");
   }
   
   VpeSetShadowed(hDoc, FALSE);
   VpeSetObjectID(hDoc, 0);
   VpePenColor(hDoc, COLOR_BLACK);
   VpeLine(hDoc, VpeGet(hDoc, VRIGHT) + 0.20, y + 0.80, VpeGet(hDoc, VRIGHT) + 2.00, y + 0.80);
   
   if (!pic_export)
      VpePrint(hDoc, VpeGet(hDoc, VRIGHT) + 0.20, y + 0.60, "[S 10 B]This object is NOT printed!");
   else
      VpePrint(hDoc, VpeGet(hDoc, VRIGHT) + 0.20, y + 0.60, "[S 10 B]This object is NOT exported!");

   // Reset Printable and Streamable properties, so
   // possibly added Charts will be printed and streamed:
   // ===================================================
   VpeSetPrintable(hDoc, TRUE);
   VpeSetStreamable(hDoc, TRUE);

   if (pic_export)
      AddCharts(hDoc);

   VpePreviewDoc(hDoc, NULL, VPE_SHOW_MAXIMIZED);
}

#else
	// Non-Windows
	// We have no preview and therefore no clickable objects, so in order to make the source code clearer, we
	// implemented a separate version.
void clickable()
{
   VpeCoord y;

   VpeHandle hDoc = VpeOpenDoc(NULL, "Clickable Report", 0);

   // We use the WIN_ANSI charset on non-Windows platforms, so we have identical PDF documents.
   // In addition, when using ASCII codes only, this make the generated PDF a bit smaller, because an extra
   // font encoding dictionary is not written.
   VpeSetCharset(hDoc, VCHARSET_WIN_ANSI);

   VpeRenderPrintBox(hDoc, 0, 0, "[S 14 C Black]x");
   LineHeight = -VpeGet(hDoc, VRENDERHEIGHT);
   
   VpeSetAutoBreak(hDoc, AUTO_BREAK_NO_LIMITS);
   VpeSetPenSize(hDoc, 0.05);
   VpePicture(hDoc, 16.50, 1.50, VFREE, VFREE, MakeFileName("..\\images\\FRUITS.BMP"));
   VpeWriteBox(hDoc, 16.50, VpeGet(hDoc, VBOTTOM), VpeGet(hDoc, VRIGHT), VFREE, "[N S 9 CE I C Blue BC Gray TO]Better Fruits Software");
   y = VpeGet(hDoc, VBOTTOM) + 0.50;
   VpePrint(hDoc, 1.50, 2.00, "[N S 26 U]Year End Results");

   VpePrint(hDoc, 1.50, 4.00, "[N S 32]Better Fruits Software");

   VpeLine(hDoc, 1.50, y, 20.00, y);
   VpeSet(hDoc, VBOTTOM, y + 0.50);
   VpeSetPenSize(hDoc, 0.03);

   double group_total, total = 0;
   char s[80], s2[80];
   int n;

   for (n = 0; n < DATA_COUNT - 1; n++)
   {
      GroupHeader(hDoc, table[n].product);
      DetailLine(hDoc, COLOR_GRAY, "Germany", table[n].germany);
      DetailLine(hDoc, COLOR_HIBLUE, "France", table[n].france);
      DetailLine(hDoc, COLOR_GRAY, "USA", table[n].usa);
      DetailLine(hDoc, COLOR_HIBLUE, "Australia", table[n].australia);
      group_total = table[n].germany +
                    table[n].france +
                    table[n].usa +
                    table[n].australia;
      GroupFooter(hDoc, group_total);
      total += group_total;
   }

   double2NumberString(total * FACTOR, s);
   sprintf(s2, "[N S 20 U]Yearly Country Sales Total: $%s", s);
   y = VpePrint(hDoc, 1.50, VpeGet(hDoc, VBOTTOM), s2);

   VpeSetFontSize(hDoc, 10);
   VpeSetBold(hDoc, TRUE);
   VpePenColor(hDoc, COLOR_BLACK);
   AddCharts(hDoc);

   ExportDocument(hDoc, "Charts.pdf");
}

#endif




// ===============================================================
//                           Bar2DDemo
// ===============================================================
void Bar2DDemo()
{
	VpeHandle hDoc = VpeOpenDoc(hMainWindow, "UDO Demo", VPE_GRIDBUTTON | VPE_NO_PAGESCROLLER);
	Bar2D = hDoc;

	VpeNoPen(hDoc);
	
	VpeWrite(hDoc, 2.00, 1.50, -17.00, VFREE, "[N CE S 16]2D-Barcodes");
	VpePrint(hDoc, 2.00, 2.80, "VPE can create the following four different 2D-barcodes:");

	VpeSetBold(hDoc, TRUE);
	VpePrint(hDoc, 2.00, 3.50, "DataMatrix");
	VpeDataMatrix(hDoc, 2.00, 4.25, -1.00, -1.00, "30Q324343430794<OQQ");

	VpePrint(hDoc, 2.00, 6.50, "MaxiCode (UPS Labels)");
	char *szStr = "[)>" "\x1e" "01" "\x1d" "9641464" 
		"\x1d" "276" "\x1d" "068" "\x1d" "1Z51146547" 
		"\x1d" "UPSN" "\x1d" "32630V" "\x1d" "327" "\x1d" 
		"\x1d" "1/1" "\x1d" "2" "\x1d" "N" "\x1d"
		"\x1d" "NEUSS" "\x1d" "\x1e" "\x04" "!!!!!!!!!!!!!!!!!!!!!!!!!!";
	VpeMaxiCode(hDoc, 2.00, 7.25, -3.00, -3.00, szStr);

	VpeMaxiCodeEx(hDoc, 6.00, 7.25, -3.00, -3.00, 2, "068107317", 840, 001, "AOE This is MaxiCode AOE", 1, 1);

	VpePrint(hDoc, 2.00, 11.50, "PDF-417");
	szStr = "01\t05\t{)>\t82\tWSP3.5.1DE\r"
		"02\t23112001\t1Z32630V6851146547\t08\tM\t9838571153\t3\t1.5\tKgs\t1.5\t\tP/P\t0.00\t\t\t\t\tDE\t\t\t\r"
		"04\tSH\tXYZ-LOGISTIK GMBH\tDUESSELDORF\t\t42134\tDE\t32630V\tC/O TRADE AG\t\t\t\tMR ERNST SCHIEFER\t+4921174648464\t\t50TH AVENUE 14\t\r"
        "04\tST\tIDEAL SOFTWARE GMBH\tNEUSS\t\t41460\tDE\t\tERFTSTR. 102 A\t\t\t\tMR Abc Xyz\t\t\t\t\r"
		"99\r";
	VpePDF417(hDoc, 2.00, 12.25, -5.20, -2.70, szStr);

	VpePrint(hDoc, 2.00, 16.00, "Aztec");
	VpeAztec(hDoc, 2.00, 16.75, -1.50, -1.50, "This is a message");

	VpeSetBold(hDoc, FALSE);
	VpePrint(hDoc, 2.00, 19.00,
		"VPE can also render (compute) the width and height of 2D-barcodes without displaying them.\n"
		"You can specify the maximum width and/or height of the rectangular space the barcode may occupy, and VPE will compute the best fitting 2D-barcode.\n"
		"Furthermore you can optionally specify a module width, which will be taken into account.");

  #ifdef CREATE_FILES
   unlink("bar-2d.vpe");
   VpeWriteDoc(hDoc, "bar-2d.vpe");
  #endif

#ifdef _WINDOWS
   VpePreviewDoc(hDoc, NULL, VPE_SHOW_MAXIMIZED);
#endif

	ExportDocument(hDoc, "Bar2D.pdf");
}


// ===============================================================
//                       Scale2GrayDemo
// ===============================================================
void Scale2GrayDemo()
{
   Msg("This demo explains the Scale-to-Gray technology of VPE Professional Edition.");

   VpeHandle hDoc = VpeOpenDoc(hMainWindow, "Scale-to-Gray", VPE_GRIDBUTTON);
   Scale2Gray = hDoc;


   // Do not allow to zoom bigger than 1:1, because the Scale2Gray
   // Images need too much memory (and CPU time) if enlarged.
   // ============================================================
#ifdef _WINDOWS
   VpeSetMaxScale(hDoc, 1.0);
   VpePreviewDoc(hDoc, NULL, VPE_SHOW_MAXIMIZED);
#endif
   VpeSet(hDoc, VRIGHTMARGIN, VpeGetPageWidth(hDoc) - 3.00);

   VpeWriteRTF(hDoc, 3.00, 1.50, VRIGHTMARGIN, VFREE,
      "\\b\\ul\\qc\\fs28 The Scale-to-Gray Technology\\b0\\fs24\\ul0\\ql\\par  \\par "
      "\\f2 A 300 or 600 DPI image scaled down to the screen preview (which is usually a 96 DPI device) is "
      "looking bad due to the nature of the scaling algorithm. "
      "The cause is, that pixels are left out when drawing such a high resolution image "
      "to the screen.\\par\\par "
      "Example:\\par If you draw a 300 DPI image to a 96 DPI screen, the image is scaled "
      "down by a factor of 96 / 300 = 0.32. This means, only 32% of all pixels are drawn, "
      "the rest is left out. (i.e. after each pixel that is drawn, 2 pixels are left out)\\par\\par\\par "
      "\\b What the Scale-to-Gray Technology does:\\b0\\par "
      "The Scale-to-Gray Technology is especially useful for displaying forms on the screen, "
      "because a grayscale image is scaled down to the low screen resolution, while the loss of "
      "visual information (the pixels, that are left out) is transformed to gray-values "
      "of different intensity (brightness). This produces perfect readability for the human eye.\\par\\par "
      "Additionally, together with the Scale-to-Gray Technology you can use 2 different "
      "images: one for the screen (preview) in low resolution, and one for printing "
      "in the original high resolution. "
      "Therefore you gain perfect readability on the screen as well as on the printouts.\\par\\par"
      "\\fi-708\\li708\\b Note:\\b0\\tab In this demo the maximum zoom factor of the preview has been limited to 1:1, "
      "because the Scale2Gray Technology is only meaningful for bitmaps which are reduced, i.e. viewed with "
	  "a scaling factor smaller than 100%. In addition Scale2Gray Images need a lot of memory (and CPU time) if enlarged.");

   VpePageBreak(hDoc);
   VpeNoPen(hDoc);
   VpePicture(hDoc, 0, 0, VFREE, VFREE, MakeFileName("..\\images\\GEW300G4.TIF"));
   VpePageBreak(hDoc);

   // Insert the scaled images now invisible for printing:
   // ====================================================
   VpeSetPrintable(hDoc, FALSE);

   // PIC_SCALE2GRAY is the choice, if you need best readability only for a 1:1
   // Preview Scaling. The scaled image is only computed once and then stored in the dynamic
   // cache (if you use PIC_KEEPIMAGE). This gives the best balance between required CPU time
   // and memory usage. The scaled image will only be re-computed, if it fell out of the
   // dynamic cache and needs to be re-created from file again.
   // =======================================================================================
   VpeSetPictureScale2Gray(hDoc, true);
   VpePicture(hDoc, 0, 0, VFREE, VFREE, MakeFileName("..\\images\\GEW300G4.TIF"));
   VpePageBreak(hDoc);

   // Now insert a Scale2GrayFloat image, which will be always computed according to the
   // preview scale:
   // ==========================================================================================
   VpeSetPictureScale2GrayFloat(hDoc, true);
   VpePicture(hDoc, 0, 0, VFREE, VFREE, MakeFileName("..\\images\\GEW300G4.TIF"));
   VpeSetPictureScale2GrayFloat(hDoc, false);

   // Insert the high resolution images now invisibly for previewing, but
   // visible for printing:
   // ===================================================================
   VpeSetPrintable(hDoc, TRUE);
   VpeSetViewable(hDoc, FALSE);
   VpeGotoPage(hDoc, 3);
   VpePicture(hDoc, 0, 0, VFREE, VFREE, MakeFileName("..\\images\\GEW300G4.TIF"));
   VpeGotoPage(hDoc, 4);
   VpePicture(hDoc, 0, 0, VFREE, VFREE, MakeFileName("..\\images\\GEW300G4.TIF"));

   // Note, that 4 images in the document share only ONE "gew300g4.tif" image in memory,
   // because of the internal cache.
   VpeSetViewable(hDoc, TRUE);
   VpeGotoPage(hDoc, 2);
   VpeSetTextColor(hDoc, COLOR_RED);
   VpeWrite(hDoc, 2.30, 4.80, -9.00, -2.00, "[B S 12]This image is scaled normally, without Scale-to-Gray.");
   VpeGotoPage(hDoc, 3);
   VpeWrite(hDoc, 2.30, 4.80, -17.00, -2.00,
      "This image is scaled with Scale-to-Gray.\n"
      "It is optimized for best readability in the 1:1 scaling of the preview.");
   VpeGotoPage(hDoc, 4);
   VpeWrite(hDoc, 2.30, 4.80, -17.00, -2.00,
      "This image is scaled with Scale-to-Gray Float.\n"
      "It offers best readability in any scaling of the preview, but "
      "needs more processor time and memory.");

  #ifdef CREATE_FILES
   unlink("scalgray.vpe");
   VpeWriteDoc(hDoc, "scalgray.vpe");
  #endif
}



#ifdef _WINDOWS
// ===============================================================
//                           UDODemo
// ===============================================================

#define UDO_OBJECT_ID 1

void UDODemo()
{
   Msg("This demo shows the capabilities of UDO's (User  Defined Objects) of VPE Professional Edition.");

   VpeHandle hDoc = VpeOpenDoc(hMainWindow, "UDO Demo", VPE_GRIDBUTTON | VPE_NO_PAGESCROLLER);
   UDO = hDoc;

   VpeSetBkgMode(hDoc, VBKG_GRD_LINE);
   VpeSetBkgGradientStartColor(hDoc, COLOR_LTGRAY);
   VpeSetBkgGradientEndColor(hDoc, COLOR_BLACK);
   VpeCreateUDO(hDoc, 1.00, 1.00, -6.00, -6.00, UDO_OBJECT_ID);

   VpeSetBkgMode(hDoc, VBKG_SOLID);
   VpeWriteRTF(hDoc, 8.00, 1.00, -10.00, VFREE,
      "To the left you see a very simple UDO (User  Defined Object). UDO's offer access to "
      "the Device Context of the Windows GDI.\\par\\par "
      "This is a very \\b powerful feature\\b0 , because it allows you to "
      "draw anything within the UDO you want.\\par\\par "
      "Additionally, UDO's provide an interface to the outside world. There are "
      "many tools and libraries out there, which offer to draw directly "
      "into a Windows Device Context (for example \\b PostScript, PDF, Charting Tools, etc.\\b0 ).\\par\\par "
      "Moreover, the \\b VPE ActiveX \\b0 offers the property 'Picture'. "
      "Therefore you can assign it the Picture property of other OLE / COM objects "
      "(and most have this property) to \\b directly visualize documents from Word, Excel, Corel Draw, etc.");

   VpeWrite(hDoc, 1.00, 8.00, -17.00, VFREE,
      "In the UDO above we simply draw two red crossing lines.\n\nThe surrounding frame "
      "and the gradient background are drawn by VPE, because a UDO inherits all properties of "
      "the Box Object. Of course you can for example turn the frame off (set PenSize = 0) and make "
      "the background transparent (set BkgMode = VBKG_TRANSPARENT).\n\n");

   VpePrint(hDoc, 1.00, nBottom(hDoc),
      "[S 11 B]Please take a look at the separate 'UDO Demo' for a sophisticated demonstration of UDO's.");

  #ifdef CREATE_FILES
   unlink("udo.vpe");
   VpeWriteDoc(hDoc, "udo.vpe");
  #endif
      
   VpePreviewDoc(hDoc, NULL, VPE_SHOW_NORMAL);

   ExportDocument(hDoc, "UDO Demo.pdf");
}



// ========================================================================
//                              UDOPaint
//
// This function is called from WndProc whenever a VPE_UDO_PAINT
// message is sent.
// ========================================================================
void UDOPaint(VpeHandle hDoc)
{
   HDC hDC;
   RECT rc;
   HPEN hpen, holdpen;

   hDC = VpeGetUDODC(hDoc);
   VpeGetUDODrawRect(hDoc, &rc);

   if (VpeGetUDOlParam(hDoc) == UDO_OBJECT_ID)
   {
      hpen = CreatePen(PS_SOLID, 1, COLOR_RED);
      holdpen = SelectPen(hDC, hpen);
      
      MoveToEx(hDC, rc.left, rc.top, NULL);
    
      LineTo(hDC, rc.right + 1, rc.bottom + 1);  // LineTo() draws a line from the current position up to,
                                                 // but not including, the specified point.
                                                 // Therefore we add 1 to each coordinate.
                                                 
      MoveToEx(hDC, rc.right, rc.top, NULL);
      LineTo(hDC, rc.left - 1, rc.bottom + 1);
      SelectObject(hDC, holdpen);
      DeleteObject(hpen);
   }
}


// ===============================================================
//                       PictureExportDemo
// ===============================================================
void PictureExportDemo()
{
   Msg("This demo exports some pages to image files and reads them into a new document.");

   VpeHandle hDoc = VpeOpenDoc(hMainWindow, "Picture Export", VPE_GRIDBUTTON);
   PicExport = hDoc;

   clickable(hDoc, 1);
}



// Is called when the "Export document as images" object
// in the PictureExport demo was clicked - export the images:
void DoExport()
{
   char s2[256];
   int  n;
   VpeHandle hDoc = PicExport;

   MessageBox(VpeGetWindowHandle(hDoc),
       "All three pages are now exported as 96 DPI 256-color images in ZIP compressed PNG format.\n"
       "Additionally we export a rectangular part of the first page to an image file.",
       szAppName, MB_OK);

   // Export as True-Color images:
   // ============================
   VpeSetPictureExportColorDepth(hDoc, PICEXP_COLOR_TRUE);
   VpeSetPictureExportDither(hDoc, PICEXP_DITHER_NONE);

   // VPE determines the file type from the filename suffix.
   // Because we use here (".1", ".2", etc.), we tell VPE the type of file:
   // =====================================================================
   VpeSetPictureType(hDoc, PIC_TYPE_PNG);

   // Set 96 DPI resolution:
   // ======================
   VpeSetPictureDefaultDPI(hDoc, 96, 96);

   TCHAR dir[MAX_PATH];
   for (n = 1; n < 4; n++)
   {
      // While this function is running, someone could have closed the parent window.
      // You can avoid this by setting the parent window to disabled,
      // but here we check for destruction:
      // ============================================================================
      if (VpeDispatchAllMessages(hDoc))
         return;
      wsprintf(dir, _T("%s\\image.%d"), GetTmpDir(), n);
      strcpy(s2, "Exporting ");
      strcat(s2, dir);
      VpeWriteStatusbar(hDoc, s2);
      VpePictureExportPage(hDoc, MakeFileName(dir), n);
   }

   // Export a rectangular area of the first page as another image:
   // =============================================================
   VpeWriteStatusbar(hDoc, "Exporting image.4");
   wsprintf(dir, _T("%s\\image.4"), GetTmpDir());
   VpePictureExport(hDoc, MakeFileName(dir), 1, 5.00, 5.00, -15.00, -10.00);

   
   MessageBox(VpeGetWindowHandle(hDoc),
       "Now all three pages are exported as 200 DPI (FAX resolution!) monochrome bitmaps in\n"
       "TIFF Fax Group 4 Multipage format (multiple pages in one file).\n"
       "During export we instruct VPE to dither them for better color-to-monochrome conversion.\n"
       "The dithering needs some processor time.\n"
       "Additionally we export a rectangular part of the first page to an image file.",
       szAppName, MB_OK);

   // VPE determines the file type from the filename suffix.
   // Because we use here (".1", ".2", etc.), we tell VPE the type of file:
   // =====================================================================
   VpeSetPictureType(hDoc, PIC_TYPE_TIFF);

   // Select TIFF FaxG4 Multipage as export format:
   // =============================================
   VpeSetTiffExportOptions(hDoc, PICEXP_TIFF_CCITTFAX4 + PICEXP_TIFF_APPEND);

   // Set 200 DPI resolution:
   // =======================
   VpeSetPictureDefaultDPI(hDoc, 200, 200);

   // By default, VPE will create a 256 color bitmap.
   // This is ok for our quality needs when we dither it down to a monochrome bitmap.
   // ===============================================================================
   VpeSetPictureExportDither(hDoc, PICEXP_DITHER_MONO);

   // Avoid, that, if program is run a second time, the newly created
   // pages are added to an existing file:
   // ===============================================================
   wsprintf(dir, _T("%s\\image.5"), GetTmpDir());
   unlink(MakeFileName(dir));
   
   for (n = 1; n < 4; n++)
   {
#ifdef _WINDOWS
      // While this function is running, someone could have closed the parent window.
      // You can avoid this by setting the parent window to disabled,
      // but here we check for destruction:
      // ============================================================================
      if (VpeDispatchAllMessages(hDoc))
         return;

      wsprintf(s2, "Exporting %s - Multipage #%d", MakeFileName(dir), n);
      VpeWriteStatusbar(hDoc, s2);
#endif
      VpePictureExportPage(hDoc, MakeFileName(dir), n);
   }

   // Export a rectangular area of the first page as another image:
   // =============================================================
   wsprintf(s2, "Exporting %s - Multipage #%d", dir, 4);
   VpeWriteStatusbar(hDoc, s2);
   VpePictureExport(hDoc, dir, 1, 5.00, 5.00, -15.00, -10.00);

   
   // Create a new document for import:
   // =================================
   MessageBox(VpeGetWindowHandle(hDoc),
         "Now the exported images are re-imported to a new document. "
         "You will notice that these are bitmaps by changing the zoom factor of the preview. "
         "Furthermore the images will have demo banners, if created with the trial version.",
         szAppName, MB_OK);

   hDocImport = VpeOpenDoc(hMainWindow, "Picture Import", VPE_GRIDBUTTON);
   VpeNoPen(hDocImport);      // no frames around imported images

   // Because VPE can not determine the file type by the filename suffix
   // (it is ".1", ".2", etc.), we tell VPE the type of file:
   // ==================================================================
   VpeSetPictureType(hDocImport, PIC_TYPE_PNG);

   for (n = 1; n < 5; n++)
   {
      // While this function is running, someone could have closed the parent window.
      // You can avoid this by setting the parent window to disabled,
      // but here we check for destruction:
      // ============================================================================
      if (VpeDispatchAllMessages(hDoc))
         return;
      wsprintf(dir, _T("%s\\image.%d"), GetTmpDir(), n);
#ifdef _WINDOWS
      strcpy(s2, "Importing ");
      strcat(s2, MakeFileName(dir));
      VpeWriteStatusbar(hDoc, s2);
#endif
      VpePicture(hDocImport, 0, 0, VFREE, VFREE, MakeFileName(dir));
      VpePageBreak(hDocImport);
   }
   

   wsprintf(dir, _T("%s\\image.5"), GetTmpDir());
   VpeSetPictureType(hDocImport, PIC_TYPE_TIFF);

   for (n = 1; n < 5; n++)
   {
#ifdef _WINDOWS
      // While this function is running, someone could have closed the parent window.
      // You can avoid this by setting the parent window to disabled,
      // but here we check for destruction:
      // ============================================================================
      if (VpeDispatchAllMessages(hDoc))
         return;
      wsprintf(s2, "Importing %s - Multipage #%d", dir, n);
      VpeWriteStatusbar(hDoc, s2);
#endif
      VpeSetPicturePage(hDocImport, n);
      VpePicture(hDocImport, 0, 0, VFREE, VFREE, MakeFileName(dir));
      if (n < 4)
         VpePageBreak(hDocImport);
   }
   
   // Reset Statusbar to default "Ready" in its specific language:
   // ============================================================
#ifdef _WINDOWS
   VpeWriteStatusbar(hDoc, NULL);
#endif

   VpeGotoPage(hDocImport, 4);
   VpePrint(hDocImport, 1.00, 12.00,
      "[C BLUE B S 12]ATTENTION: This page demonstrates that VPE is also capable to export a rectangular area of a page, instead of a whole page");
   VpeGotoPage(hDocImport, 8);
   VpePrint(hDocImport, 1.00, 12.00,
      "[C BLUE B S 12]ATTENTION: This page demonstrates that VPE is also capable to export a rectangular area of a page, instead of a whole page");

   VpeGotoPage(hDocImport, 5);
   VpePrint(hDocImport, 1.00, 1.00,
      "[C RED B S 12]ATTENTION: The following images are dithered with the "
      "Floyd-Steinberg algorithm to 200 DPI. View them with a zoom factor of x2, which equals nearly 200 DPI!");

   VpePreviewDoc(hDocImport, NULL, VPE_SHOW_MAXIMIZED);
}


// ===============================================================
//                     DrawGradientRect
// ===============================================================
typedef struct
{
   LONG red, green, blue, deltared, deltagreen, deltablue;
   int bandcount;
} GRADIENT_SCHEME;

#define STEPDIVISOR 1024
#define GRADIENTDIVISOR 1024

void DrawGradientRect(HDC hdc, RECT &rectClient)
{
   RECT rectFill;
   HBRUSH hBrush;
   int fStep, iOnBand;
   GRADIENT_SCHEME scheme = { 0, 0, 261120,  0,  0, -1024, 255 };

   fStep = MulDiv(rectClient.bottom - rectClient.top, STEPDIVISOR, scheme.bandcount + 1);
   for (iOnBand = 0; iOnBand < scheme.bandcount + 1; iOnBand++) 
   {
         // Set the location of the current band
         SetRect(&rectFill,
               rectClient.left,                                           // Upper left X
               MulDiv(iOnBand, fStep, STEPDIVISOR) + rectClient.top,      // Upper left Y
               rectClient.right,                                          // Lower right X
               MulDiv(iOnBand + 1, fStep, STEPDIVISOR) + rectClient.top); // Lower right Y

         // Create a brush with the appropriate color for this band
         hBrush = CreateSolidBrush(RGB(scheme.red / GRADIENTDIVISOR, scheme.green / GRADIENTDIVISOR, scheme.blue / GRADIENTDIVISOR));
      scheme.red   += scheme.deltared;
      scheme.green += scheme.deltagreen;
      scheme.blue  += scheme.deltablue;
      FillRect(hdc, &rectFill, hBrush);
      DeleteObject(hBrush);
   }
}




// ========================================================================
//                       Paint the Demo Window itself
// ========================================================================
HFONT create_font(char *name, int pt_size, BYTE italic)
{
   return CreateFont(pt_size, 0,
               0, 0,
               FW_DONTCARE,     // bold ? FW_BOLD : FW_DONTCARE,
               italic,
               0,               // underlined,
               0, 0, 0, 0, 0, 0, name);
}


void Paint(HWND hwnd)
{
   PAINTSTRUCT ps;
   HDC hdc;
   RECT rectClient;
   HFONT hfont, holdfont;
   char *s, *SUBTITLE;
   int w, h;
   char Product[80];

   wsprintf(Product, PRODUCT, VpeGetVersion() >> 8, VpeGetVersion() & 0xff);
   
   hdc = BeginPaint(hwnd, &ps);
   GetClientRect(hwnd, &rectClient);
   DrawGradientRect(hdc, rectClient);

   SetTextColor(hdc, 0xffffff);
   SetBkMode(hdc, TRANSPARENT);

   hfont = create_font(TIMES, -36, 1);
   holdfont = SelectFont(hdc, hfont);
   SetTextColor(hdc, 0);
   TextOut(hdc, 17, 17, Product, strlen(Product));
   SetTextColor(hdc, 0xffffff);
   TextOut(hdc, 15, 15, Product, strlen(Product));
   SelectFont(hdc, holdfont);
   DeleteFont(hfont);

   switch (VpeGetEdition())
   {
    case VEDITION_STANDARD: SUBTITLE = "Standard Edition"; break;
    case VEDITION_ENHANCED: SUBTITLE = "Enhanced Edition"; break;
    case VEDITION_PROFESSIONAL: SUBTITLE = "Professional Edition"; break;
    case VEDITION_ENTERPRISE: SUBTITLE = "Enterprise Edition"; break;
    case VEDITION_INTERACTIVE: SUBTITLE = "Interactive Edition"; break;
   }

   hfont = create_font(HELVETICA, -18, 0);
   holdfont = SelectFont(hdc, hfont);
   SetTextColor(hdc, 0);
   TextOut(hdc, 16, 61, SUBTITLE, strlen(SUBTITLE));
   SetTextColor(hdc, 0xffffff);
   TextOut(hdc, 15, 60, SUBTITLE, strlen(SUBTITLE));
   SelectFont(hdc, holdfont);
   DeleteFont(hfont);
         

   hfont = create_font(HELVETICA, -18, 0);
   holdfont = SelectFont(hdc, hfont);
   s = "Copyright  2008 IDEAL Software";
 #if defined(__WIN32__) || defined(WIN32) || defined(__NT__)
      SIZE p;
      GetTextExtentPoint32(hdc, s, strlen(s), &p);
      w = p.cx;
      h = p.cy;
 #else
      DWORD wh = GetTextExtent(hdc, s, strlen(s));
      w = LOWORD(wh);
      h = HIWORD(wh);
 #endif
   TextOut(hdc, 15, rectClient.bottom - 3*h, s, strlen(s));
   s = "";
   TextOut(hdc, 15+w, rectClient.bottom - 3*h - h/3, s, strlen(s));
   s = "All rights reserved.";
   TextOut(hdc, 15, rectClient.bottom - 2*h, s, strlen(s));
   SelectFont(hdc, holdfont);
   DeleteFont(hfont);

   EndPaint(hwnd, &ps);
}




// ========================================================================
//                              WndProc
// ========================================================================
long CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
   static HINSTANCE  hInstance;
   int    i;
   
   switch (message)
   {
    case WM_CREATE:
       hMainWindow = hwnd;
       hInstance = ((LPCREATESTRUCT) lParam)->hInstance;
#ifdef PROFILE
	   RTFMeasure();
#endif
       return 0;


    case WM_PAINT:
       if (!RTF)
       {
          Paint(hwnd);
          return 0;
       }
       break;


    case WM_COMMAND:
       switch (wParam)
       {
        case CM_RTF:
           RTFDemo();
           EnableMenuItem(hMenu, CM_RTF, MF_GRAYED);
           break;

        case CM_CLICKABLE:
           clickable();
           EnableMenuItem(hMenu, CM_CLICKABLE, MF_GRAYED);
           break;
           
        case CM_BAR2D:
           Bar2DDemo();
           EnableMenuItem(hMenu, CM_BAR2D, MF_GRAYED);
           break;

        case CM_UDO:
           UDODemo();
           EnableMenuItem(hMenu, CM_UDO, MF_GRAYED);
           break;

        case CM_SCALE2GRAY:
           Scale2GrayDemo();
           EnableMenuItem(hMenu, CM_SCALE2GRAY, MF_GRAYED);
           break;

        case CM_PICEXPO:
           PictureExportDemo();
           EnableMenuItem(hMenu, CM_PICEXPO, MF_GRAYED);
           break;

        case CM_EXIT:
           PostMessage(hwnd, WM_CLOSE, 0, 0);
           break;
       }
       return 0;


    case WM_SIZE:
       if (RTF)   // this is an embedded window
       {
          MoveWindow(VpeGetWindowHandle(RTF), 0, 0, LOWORD(lParam), HIWORD(lParam), TRUE);
       }
       return 0;


    case WM_SETFOCUS:
       if (RTF)
       {
			 // this is an embedded window, forward the focus to the preview, so
			 // it receives keyboard and mouse-wheel messages
          SetFocus(VpeGetWindowHandle(RTF));
       }
       return 0;


    case WM_CLOSE:
       if (!VpeCloseDoc(RTF) ||     // VpeCloseDoc() allows that the hDoc parameter is NULL
           !VpeCloseDoc(Clickable) ||
           !CloseAllCharts() ||
		   !VpeCloseDoc(Bar2D) ||
           !VpeCloseDoc(UDO) ||
           !VpeCloseDoc(Scale2Gray) ||
           !VpeCloseDoc(PicExport) ||
           !VpeCloseDoc(hDocImport))
       {
          Msg("The application cannot terminate until all jobs have finished printing.");
          return 0;
       }
       DestroyWindow(hwnd);
       return 1;
              
           
    case WM_DESTROY:
       if (RTF)                 // embedded window needs to be destroyed in advance
          VpeCloseDoc(RTF);     // because of MAPI problems (process will live on)
       PostQuitMessage(0);
       return 0;
   }



   // Check messages for VPE document:
   // ================================
   switch (VpeMapMessage(NULL, message))        // we use NULL, because all documents are
   {                                             // created with dynamic message mapping, i.e. VPE_FIXED_MESSAGES is not used
    case VPE_DESTROYWINDOW:
       if (RTF == (VpeHandle)lParam)
       {
          RTF = 0;
          EnableMenuItem(hMenu, CM_RTF, MF_ENABLED);
       }
       else if (Clickable == (VpeHandle)lParam)
       {
          Clickable = 0;
          EnableMenuItem(hMenu, CM_CLICKABLE, MF_ENABLED);
          for (i = 0; i < DATA_COUNT; i++)
          {
             if (table[i].hDoc)
                VpeCloseDoc(table[i].hDoc);
          }
       }
       else if (Bar2D == (VpeHandle)lParam)
       {
          Bar2D = 0;
          EnableMenuItem(hMenu, CM_BAR2D, MF_ENABLED);
       }
       else if (UDO == (VpeHandle)lParam)
       {
          UDO = 0;
          EnableMenuItem(hMenu, CM_UDO, MF_ENABLED);
       }
       else if (Scale2Gray == (VpeHandle)lParam)
       {
          Scale2Gray = 0;
          EnableMenuItem(hMenu, CM_SCALE2GRAY, MF_ENABLED);
       }
       else if (PicExport == (VpeHandle)lParam)
       {
          PicExport = 0;
          if (hDocImport)
             VpeCloseDoc(hDocImport);
          EnableMenuItem(hMenu, CM_PICEXPO, MF_ENABLED);
       }
       else if (hDocImport == (VpeHandle)lParam)
       {
          hDocImport = 0;
       }
       else
       {
          // One of the Chart Previews was destroyed:
          // ========================================
          for (i = 0; i < DATA_COUNT; i++)
          {
             if (table[i].hDoc == (VpeHandle)lParam)
                table[i].hDoc = 0;
          }
       }
       return 0;

                              
    case VPE_OBJECTCLICKED:
       if (VpeGetObjectID((VpeHandle)lParam) < 100)
       {
           // A table in the Clickable demo was clicked
           // generate the corresponding chart in a separate document:
           // ========================================================
           show_chart((int)VpeGetObjectID((VpeHandle)lParam));    // show in separate window
       }
       else if (VpeGetObjectID((VpeHandle)lParam) == 100)
       {
          // The logo in the Clickable demo was clicked generate a message:
          // ================================================================
          MessageBox(VpeGetWindowHandle((VpeHandle)lParam), "Thank you for chosing 'Better Fruits Software'.", szAppName, MB_OK);
       }
       else if (VpeGetObjectID((VpeHandle)lParam) == 101)
       {
          // The "Add Charts to document" object in the Clickable
          // demo was clicked - add all charts to the document:
          // ====================================================
          if (!AddedCharts)
          {
             if (AddCharts(Clickable))
             {
                AddedCharts = true;
                MessageBox(VpeGetWindowHandle((VpeHandle)lParam), "Charts added to document.", szAppName, MB_OK);

				ExportDocument(Clickable, "Charts.pdf");

               #ifdef CREATE_FILES
                unlink("charts.vpe");
                VpeWriteDoc((VpeHandle)lParam, "charts.vpe");
               #endif
             }
          }
          VpeGotoVisualPage(Clickable, 2);
       }
       else if (VpeGetObjectID((VpeHandle)lParam) == 200)
       {
          // The "Export document as images" object in the PictureExport
          // demo was clicked - export the images:
          // ===========================================================
          if (!hDocImport)
             DoExport();
          else
             SetFocus(VpeGetWindowHandle(hDocImport));
       }
       return 0;
         
        
    case VPE_UDO_PAINT:
       UDOPaint((VpeHandle)lParam);
       return 1;		// indicate, that the event has been processed by this application


    case VPE_HELP:
       MessageBox(hwnd, "User requested help!", "Note:", MB_OK);
       break;
   }

   return DefWindowProc (hwnd, message, wParam, lParam) ;
}


// ========================================================================
//                              WinMain
// ========================================================================
int PASCAL WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance,
                    LPSTR , int nCmdShow)
{
   MSG         msg ;
   HWND        hwnd ;
   WNDCLASS    wndclass ;

   setlocale(LC_ALL, "");
   Locale = *localeconv();

   if (!hPrevInstance)
   {
      wndclass.style         = CS_HREDRAW | CS_VREDRAW;
      wndclass.lpfnWndProc   = WndProc ;
      wndclass.cbClsExtra    = 0 ;
      wndclass.cbWndExtra    = 0 ;
      wndclass.hInstance     = hInstance ;
      wndclass.hIcon         = LoadIcon (hInstance, "APP_ICON");
      wndclass.hCursor       = LoadCursor (NULL, IDC_ARROW) ;
      wndclass.hbrBackground = GetStockBrush (NULL_BRUSH) ;
      wndclass.lpszMenuName  = NULL;
      wndclass.lpszClassName = szAppName;
      RegisterClass(&wndclass) ;
   }

   hMenu = LoadMenu(hInstance, MAKEINTRESOURCE(MENU_1));

   hwnd = CreateWindow(szAppName, szAppName,
                       WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN,
                       CW_USEDEFAULT, CW_USEDEFAULT,
                       CW_USEDEFAULT, CW_USEDEFAULT,
                       NULL, hMenu, hInstance, NULL) ;

     
   ShowWindow (hwnd, nCmdShow) ;
   UpdateWindow (hwnd) ;

   while (GetMessage (&msg, NULL, 0, 0))
   {
           TranslateMessage (&msg) ;
           DispatchMessage (&msg) ;
   }

   return msg.wParam;
}

#else

// ========================================================================
//                              Non-Windows main()
// ========================================================================
int main(int argc, const char **argv)
{
	setlocale(LC_ALL, "");
	Locale = *localeconv();

	struct stat st_buf;
	if (stat("../images", &st_buf) != 0)
	{
		const char *exe_name = "";
		int i = strlen(argv[0]);
		while (i >= 0 && argv[0][i] != '/')
			i--;
		if (i >= 0)
			exe_name = &argv[0][i + 1];
		printf("\nPlease run this demo from within its original directory,\ni.e. with './%s'\n\n", exe_name);
		return 1;
	}

	RTFDemo();
	clickable();
	Bar2DDemo();

	// UDO's and Picture Export not supported on Non-Windows platforms

	return 0;
}
#endif
